//********************************************************************
//*** ResourcePool/ResourcePoolStats.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: ResourcePoolStats.cc,v 1.2 2005/09/20 22:19:09 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "ResourcePoolStats.hh"

namespace fatalmind {

ResourcePoolStats::ResourcePoolStats() 
: getCalls_(0)
, waitEvents_(0)
, failCalls_(0)
, usedSize_(0)
, freeSize_(0)
{
}

ResourcePoolStats::ResourcePoolStats(
	  const ResourcePoolStats& rho
	, int usedSize
	, int freeSize
):getCalls_(rho.getCalls())
, waitEvents_(rho.waitEvents())
, failCalls_(rho.failCalls())
, usedSize_(usedSize)
, freeSize_(freeSize)
{
}

ResourcePoolStats::~ResourcePoolStats() {
}

unsigned long int ResourcePoolStats::getCalls() const {
	return getCalls_;
}

unsigned long int ResourcePoolStats::waitEvents() const {
	return waitEvents_;
}

unsigned long int ResourcePoolStats::failCalls() const {
	return failCalls_;
}

int ResourcePoolStats::usedSize() const {
	return usedSize_;
}

int ResourcePoolStats::freeSize() const {
	return freeSize_;
}

float ResourcePoolStats::waitRatio() const {
	return (((float)waitEvents_)/getCalls_)*100;
}

float ResourcePoolStats::failRatio() const {
	return (((float)failCalls_)/getCalls_)*100;
}

ResourcePoolStats& ResourcePoolStats::operator-=(const ResourcePoolStats& rho) {
	getCalls_   -= rho.getCalls_;
	waitEvents_ -= rho.waitEvents_;
	failCalls_  -= rho.failCalls_;

	return *this;
}

ResourcePoolStats& ResourcePoolStats::operator+=(const ResourcePoolStats& rho) {
	getCalls_   += rho.getCalls_;
	waitEvents_ += rho.waitEvents_;
	failCalls_  += rho.failCalls_;

	usedSize_   += rho.usedSize_;
	freeSize_   += rho.freeSize_;

	return *this;
}

struct privateTypeForMinipulator1 {
	float value;
};

privateTypeForMinipulator1 putPercentage(float val) {
	privateTypeForMinipulator1 x;
	x.value = val;
	return x;
}

std::ostream& operator<<(std::ostream& os, const privateTypeForMinipulator1& a) {
	int oldWidth = os.width();
	int oldPrecision = os.precision();	
	os.width(5);
	os.precision(2);
	os << a.value;
	os.width(oldWidth);
	os.precision(oldPrecision);
	return os;
}



void ResourcePoolStats::getCall_() {
	++getCalls_;
}

void ResourcePoolStats::waitEvent_() {
	++waitEvents_;
}

void ResourcePoolStats::failCall_() {
	++failCalls_;
}

} // namespace fatalmind

using namespace fatalmind;

std::ostream& operator<<(std::ostream& os, const ResourcePoolStats& stats) {
	os << "ResourcePoolStats[get=" << stats.getCalls() 
		<< "; wait=" << stats.waitEvents() << " (" << putPercentage(stats.waitRatio()) << "%)"
		<< "; fail=" << stats.failCalls()  << " (" << putPercentage(stats.failRatio()) << "%)"
		<< "; usedSize=" << stats.usedSize() 
		<< "; freeSize=" << stats.freeSize()
		<< "]";
	return os;
}

