//********************************************************************
//*** ResourcePool/SQL/BindOut.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: BindOut.cc,v 1.13 2005/09/20 22:18:07 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include <string>
#include <sstream>
#include "BindOut.hh"

namespace fatalmind {
namespace SQL {

template<class VBC, template<class> class CT>
BindOut<VBC, CT>::~BindOut() {
}

template<class VBC, template<class> class CT>
template<typename T>
void BindOut<VBC, CT>::bindout(const unsigned int pos, T& val) {
	gc_ptr<VBC> ptr(new CT<T>(val));
	_outparam.resize(std::max<int>(pos + 1, _outparam.size()));
	_outparam[pos] = ptr;
}

template<class VBC, template<class> class CT>
bool BindOut<VBC, CT>::isnull(const unsigned int pos) const {
	if (pos < _outparam.size()) {
		return (_outparam[pos]->isnull());
	} else {
		std::ostringstream os;
		os << "bindout(" << pos << ", ...) has not been called";
		throw SQLException(os.str(), "07001", "UKN", false);
	}
}

template<class VBC, template<class> class CT>
VBC& BindOut<VBC, CT>::getOutParam(const unsigned int pos) {
	if (pos < _outparam.size()) {
		if (!_outparam[pos]->isnull()) {
			return *(_outparam[pos]);
		} else {
			std::ostringstream os;
			os << "getOutParam(" << pos << ") for NULL column called";
			throw SQLException(os.str(), "07001", "UKN", false);
		}
	} else {
		std::ostringstream os;
		os << "bindout(" << pos << ", ...) has not been called";
		throw SQLException(os.str(), "07001", "UKN", false);
	}
}

} // namespace SQL
} // namespece fatalmind
