//********************************************************************
//*** ResourcePool/SQL/RTSQLExecute.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLExecute.cc,v 1.15 2005/08/21 07:48:03 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "config.h"
#include "ResourcePool/SQL/ResourcePool.hh"
#include "ResourcePool/SQL/RTSQLExecute.hh"
#include "ResourcePool/SQL/RTSQLExecute_Impl.hh"

#include <iostream>
#include <sstream>

namespace fatalmind {
namespace SQL {

template<class C, class TM, class RTSQLC>
class SQLExecuteWrapper
: public C
{
    public:
		SQLExecuteWrapper(RTSQLC& rtc, const std::string& sql)
		: C(sql)
		{
		}
};

template<class TM>
struct SQLExecuteCommandSelector {
	typedef RTSQLExecute_Impl<TM> Impl;
	typedef RTSQLExecute<TM> RTC;

	template<class PT>
	struct Specialized {
		typedef RTSQLExecute_SpecializedImpl<
			  TM
			, RTSQLExecute_Impl
			, PT
			, SQLExecuteWrapper<typename PT::command_t::SQLExecute, TM, RTSQLExecute<TM> >
			, RTSQLExecute<TM>
		> Impl;
	};
};

template<class TM>
RTSQLExecute<TM>::RTSQLExecute(const ResourcePool<ResourceType<TM> >& p, const std::string& a_sql)
: _realSQLExecute(
	RTSQLExecute_Impl<TM>::template newRealCommand<
		SQLExecuteCommandSelector<TM>
	>(
		  Command<ResourceType<TM> >::getFactory(p).getDatabaseType()
		, *this
		, a_sql
	)
)
{
}

template<class TM>
RTSQLExecute<TM>::~RTSQLExecute() {
}

template<class TM>
template<class T>
void RTSQLExecute<TM>::bind(const unsigned int pos, const T& val, bool nul) {
	_realSQLExecute->bind(pos, val, nul);
}

template<class TM>
void RTSQLExecute<TM>::execute(typename ResourceType<TM>::plainresource_t& hand) {
	_realSQLExecute->execute(hand);
}

template class RTSQLExecute<DefaultThreadedModel >;

template void RTSQLExecute<DefaultThreadedModel >::bind<char>
	(const unsigned int pos, const char& val, bool nul);
template void RTSQLExecute<DefaultThreadedModel >::bind<signed char>
	(const unsigned int pos, const signed char& val, bool nul);
template void RTSQLExecute<DefaultThreadedModel >::bind<unsigned char>
	(const unsigned int pos, const unsigned char& val, bool nul);

template void RTSQLExecute<DefaultThreadedModel >::bind<short>
	(const unsigned int pos, const short& val, bool nul);
template void RTSQLExecute<DefaultThreadedModel >::bind<int>
	(const unsigned int pos, const int& val, bool nul);
template void RTSQLExecute<DefaultThreadedModel >::bind<long int>
	(const unsigned int pos, const long int& val, bool nul);

template void RTSQLExecute<DefaultThreadedModel >::bind<unsigned short>
	(const unsigned int pos, const unsigned short& val, bool nul);
template void RTSQLExecute<DefaultThreadedModel >::bind<unsigned int>
	(const unsigned int pos, const unsigned int& val, bool nul);
template void RTSQLExecute<DefaultThreadedModel >::bind<unsigned long int>
	(const unsigned int pos, const unsigned long int& val, bool nul);

template void RTSQLExecute<DefaultThreadedModel >::bind<double>
	(const unsigned int pos, const double& val, bool nul);
template void RTSQLExecute<DefaultThreadedModel >::bind<float>
	(const unsigned int pos, const float& val, bool nul);
template void RTSQLExecute<DefaultThreadedModel >::bind<long double>
	(const unsigned int pos, const long double& val, bool nul);

template void RTSQLExecute<DefaultThreadedModel >::bind<std::string>
	(const unsigned int pos, const std::string& val, bool nul);

} // namespace SQL
} // namespace fatalmind
