//********************************************************************
//*** ResourcePool/mysql/ResourcePool.hh
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: ResourcePool.hh,v 1.11 2005/03/13 14:57:19 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */


#ifndef RESOURCEPOOL_MYSQL_RESOURCEPOOL_HH
#define RESOURCEPOOL_MYSQL_RESOURCEPOOL_HH

#ifndef RESOURCEPOOL_MYSQL_RESOURCE_HH
#include "ResourcePool/mysql/Resource.hh"
#endif

#ifndef RESOURCEPOOL_MYSQL_FACTORY_HH
#include "ResourcePool/mysql/Factory.hh"
#endif

#ifndef RESOURCEPOOL_RESOURCEPOOL_HH
#include "ResourcePool/ResourcePool.hh"
#endif

#ifndef RESOURCEPOOL_MYSQL_MYSQLEXCEPTION_HH
#include "MysqlException.hh"
#endif

namespace fatalmind {

// forward declarations
/**
 * the fatalmind::mysql namespace contains
 * all classes required to implement the mysql
 * ResourceType.
 */
namespace mysql {

class SQLSelect;
class SQLExecute;
class SQLSelectRow;

} // namespace mysql

/**
 * the fatalmind::ResourceType namespace contains\
 * all available types for ResourcePool.
 */
namespace ResourceType {

struct mysqlCommands {
	mysqlCommands(const mysqlFactory&) {
	};

	typedef fatalmind::mysql::SQLExecute   SQLExecute;
	typedef fatalmind::mysql::SQLSelect    SQLSelect;
	typedef fatalmind::mysql::SQLSelectRow SQLSelectRow;
};

/**
 * The resource type to access mysql databases.
 */
typedef PoolType<mysqlFactory, mysqlResource, MYSQL, mysqlCommands> mysql;

} // namespace ResourceType

namespace mysql {

typedef fatalmind::ResourceType::mysql ResourceType;

} // namespace mysql
} // namespace fatalmind

#ifndef RESOURCEPOOL_MYSQL_SQLSELECT_HH
#include "SQLSelect.hh"
#endif

#ifndef RESOURCEPOOL_MYSQL_SQLEXECUTE_HH
#include "SQLExecute.hh"
#endif

#ifndef RESOURCEPOOL_MYSQL_SQLSelectRow_HH
#include "SQLSelectRow.hh"
#endif

#endif
