//********************************************************************
//*** Thread/ThreadRunnerInterface.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: TaskRunnerInterface.cc,v 1.1 2005/05/10 19:14:20 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "Thread/TaskRunnerInterface.hh"
#include "Thread/WaitObject.hh"
#include "mm/gc_ptr.hh"
#include "mm/CommonPtr.hh"

namespace fatalmind {

/*
 *
 * Implementation of WaitObject
 *
 */

WaitObject::WaitObject()
: _condvar()
, _done(false)
{
}

WaitObject::~WaitObject()
{
}

void WaitObject::done()
{
	sync_t lk(_condvar);
	_done = true;
	_condvar.broadcast();
}

void WaitObject::wait()
{
	sync_t lk(_condvar);
	while (! _done) {
		_condvar.wait();
	}
}

/*
 *
 * Implementation of ID
 *
 */

TaskRunnerInterface::ID::ID()
: _wo()
{
}

TaskRunnerInterface::ID::ID(const ID& rho)
: _wo(rho._wo)
{
}


TaskRunnerInterface::ID::ID(gc_ptr<WaitObject, MultiThreadedModel<> >& wo)
: _wo(wo)
{
}

TaskRunnerInterface::ID::~ID() throw () {
}

TaskRunnerInterface::ID& TaskRunnerInterface::ID::operator=(const ID& rho)
{
	_wo = rho._wo;
	return *this;
}

void TaskRunnerInterface::ID::wait()
{
	_wo->wait();
}

void TaskRunnerInterface::ID::detach()
{
	_wo.reset();
}






} // namespace fatalmind
