//********************************************************************
//*** Thread/Thread.hh
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: Thread.hh,v 1.15 2005/08/21 08:53:49 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef INCLUDED_THREAD_H
#define INCLUDED_THREAD_H

#include <memory>

// TODO should be hidden, currently reqd for pthread_t member
#include <pthread.h>

#ifndef INCLUDED_TASK_HH
#include "Thread/Task.hh"
#endif

#include "mm/gc_ptr.hh"
#include "mm/CommonPtr.hh"

namespace fatalmind {

/**
 * A representation of a Thread. 
 *
 * A Thread can execute a single Task, when the Task has finished,
 * the thread will terminate.
 *
 * If you need to run multiple tasks and don't want to create 
 * multiple Threads for that you can use a TaskRunner or TaskRunnerPool.
 */
class Thread {
	public:
		Thread();
		~Thread();

		template<class D>
		inline void start(std::auto_ptr<D> a) {
			start(std::auto_ptr<Task>(a));
		}
		template<class D>
		inline void start(gc_ptr<D, MultiThreadedModel<> > a) {
			start(gc_ptr<Task, MultiThreadedModel<> >(a));
		}
		void start(std::auto_ptr<Task>);
		void start(gc_ptr<Task, MultiThreadedModel<> >);

		void detach();
		void join();
		static void yield();

	protected:
		void start(Task*);
		static void* jumpinpoint(void* arg);

		CommonPtr<Task, MultiThreadedModel<> > d_task;

		bool		d_started;
		pthread_t	d_threadid;
};

} // namespace fatalmind
#endif
